#!/usr/bin/perl -w
# $Id: bootstrap_candidate_precursors.pl,v 1.14 2010/07/07 12:26:36 rnl14480 Exp $

use strict;
use lib qw(../lib/);

use BAOBAB::ChromosomeWrapper;
use BAOBAB::Chromosome;
use BAOBAB::OrganismWrapper;
use BAOBAB::Organism;
use BAOBAB::PrecursorWrapper;
use BAOBAB::Precursor;
use BAOBAB::Fasta;
use BAOBAB::Wrapper;
use DBI;

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

die "Usage: $0 data-dir\n" unless scalar(@ARGV) >= 1;
my $data = shift;
my @organisms = qw(ada aga planted_metazoa dmel);

foreach my $org (@organisms) {

	print STDERR "\nProcessing organism $org...\n";
	my $fasta   = get_fasta($org);
	my $results = get_results($org);
	my $handler = get_handler($org);

	print STDERR "Processing Fasta file : $fasta\n";
	my $fobj = BAOBAB::Fasta->new(
		filename       => $fasta,
		header_handler => $handler
	);

	my $rfile;
	open( $rfile, $results ) or die "Cannot open file $results : $!\n";

	my $ow = BAOBAB::OrganismWrapper->new( dbh => $dbh );
	$ow->read( shortname => $org );

	print STDERR "\n";
	my $count = 0;
	while (<$rfile>) {
		chomp;
		my @fields = split /:/;
		my $chr_nr = $fields[0];
		my $strand = $fields[1];
		my $start  = $fields[2];
		my $mfe    = $fields[3];
		my $stop   = $start + $fields[5] + $fields[6] + $fields[7] - 1;

		my @seqobjs = $fobj->sequences;
		my $seqobj  = $seqobjs[$chr_nr];
		my $chr     = $seqobj->id;

		my $cw = BAOBAB::ChromosomeWrapper->new( dbh => $dbh );
		$cw->read( name => $chr, oid => $ow->wrap->id );

		my $pseq = $fobj->get_region(
			chr    => $chr,
			offset => $start,
			len    => $stop - $start + 1,
			strand => $strand
		);

		warn "Sequence length ("
		  . ( $stop - $start + 1 )
		  . ") exceeds maximum length 1000\n"
		  if $stop - $start + 1 > 1000;

		my $p = BAOBAB::Precursor->new(
			chromosome => $cw->wrap,
			sequence   => $pseq,
			structure  => undef,
			mfe        => $mfe,
			start      => $start,
			stop       => $stop,
			strand     => $strand,
			category   => 'candidate',
			score      => 0.0,
			marked     => 0,
			no_cache   => 1
		);
		$p->structure;

		my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh, object => $p );
		$pw->write;
	}
}

$dbh->disconnect;

sub get_fasta {
	my $o    = shift;
	my %data = (
		ada                    => $data . q(darlingi.nfa),
		aga                    => $data . q(aga.nfa),
		planted_metazoa        => $data . q(planted_metazoa.nfa),
		dmel                   => $data . q{dmel.nfa},
		random_ada             => $data . q{random_ada.nfa},
		random_aga             => $data . q{random_aga.nfa},
		random_planted_metazoa => $data . q{random_planted_metazoa.nfa},
		random_dmel            => $data . q{random_dmel.nfa},
	);

	return $data{$o};
}

sub get_results {
	my $o    = shift;
	my %data = (
		ada                    => $data . q(darlingi.fsl),
		aga                    => $data . q(aga.fsl),
		planted_metazoa        => $data . q(planted_metazoa.fsl),
		dmel                   => $data . q{dmel.fsl},
		random_ada             => $data . q{random_ada.fsl},
		random_aga             => $data . q{random_aga.fsl},
		random_planted_metazoa => $data . q{random_planted_metazoa.fsl},
		random_dmel            => $data . q{random_dmel.fsl},
	);

	return $data{$o};
}

sub get_handler {

	my $o    = shift;
	my %data = (
		ada => sub { $_ = shift; s/\s//g; return $_; },
		aga                    => sub { ( split /:/, shift )[1] },
		planted_metazoa        => sub { return 'U' },
		random_ada             => sub { return 'U' },
		random_aga             => sub { return 'U' },
		random_planted_metazoa => sub { return 'U' },
		random_dmel            => sub { return 'U' },

		#		hsa_mit                => sub { return 'C' },
		dmel => sub {
			$_ = shift;
			if (/^arm_(\S+)/) {
				return $1;
			}
			elsif (/^Arm(\S+)/) {
				return $1;
			}
			elsif (/^(\S+)/) {
				return $1;
			}
			else {
				return '??';
			}
		}
	);

	return $data{$o};
}

1;
