#!/usr/bin/perl -w
# $Id: bootstrap_known_precursors.pl,v 1.9 2010/07/07 12:33:39 rnl14480 Exp $

use strict;
use lib qw(../lib/);

use BAOBAB::ChromosomeWrapper;
use BAOBAB::Chromosome;
use BAOBAB::OrganismWrapper;
use BAOBAB::Organism;
use BAOBAB::PrecursorWrapper;
use BAOBAB::Precursor;
use BAOBAB::Mature;
use BAOBAB::MatureWrapper;
use BAOBAB::Fasta;
use BAOBAB::Wrapper;
use DBI;
use MirBase::ParseName qw(parse_name);

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

die "Usage: $0 data-dir\n" unless scalar(@ARGV) >= 1;

my $data = shift;

my @organisms = qw( aga dmel planted_metazoa );

foreach my $org (@organisms) {

	print "Processing organism $org...\n";
	my $fasta   = get_fasta($org);
	my $loc     = get_loc($org);
	my $handler = get_handler($org);

	print STDERR "Processing Fasta file : $fasta\n";
	my $fobj = BAOBAB::Fasta->new(
		filename       => $fasta,
		header_handler => $handler
	);

	my $locfile;
	open( $locfile, $loc ) or die "Cannot open file $loc : $!\n";

	my $ow = BAOBAB::OrganismWrapper->new( dbh => $dbh );
	$ow->read( shortname => $org );

	while (<$locfile>) {
		chomp;
		my @fields      = split /:/;
		my $chr_nr      = $fields[1];
		my $strand      = $fields[2];
		my $start       = $fields[3];
		my $stop        = $fields[6];
		my $designation = $fields[7];

		my @seqobjs = $fobj->sequences;
		my $seqobj  = $seqobjs[$chr_nr];
		my $chr     = $seqobj->id;

		#print STDERR "chr_nr = $chr_nr chr_name = $chr\n";

		my $cw = BAOBAB::ChromosomeWrapper->new( dbh => $dbh );
		$cw->read( name => $chr, oid => $ow->wrap->id );

		#print STDERR "chr_id = ".$cw->wrap->id."\n";

		my $pseq = $fobj->get_region(
			chr    => $chr,
			offset => $start,
			len    => $stop - $start + 1,
			strand => $strand
		);

		warn "Sequence length ("
		  . ( $stop - $start + 1 )
		  . ") exceeds maximum length 1000\n"
		  if $stop - $start + 1 > 1000;

		my $p = BAOBAB::Precursor->new(
			chromosome  => $cw->wrap,
			sequence    => $pseq,
			structure   => undef,
			mfe         => undef,
			start       => $start,
			stop        => $stop,
			strand      => $strand,
			category    => 'known',
			score       => 0.0,
			marked      => 0,
			designation => $designation,
			no_cache    => 1
		);
		$p->structure;

		my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh, object => $p );
		$pw->write;
	}
}

$dbh->disconnect;

sub get_fasta {
	my $o    = shift;
	my %data = (
		aga             => $data . q(aga.nfa),
		planted_metazoa => $data . q(planted_metazoa.nfa),
		dmel            => $data . q(dmel.nfa),
	);

	return $data{$o};
}

sub get_loc {
	my $o    = shift;
	my %data = (
		aga             => $data . q(aga.loc),
		planted_metazoa => $data . q(planted_metazoa.loc),
		dmel            => $data . q(dmel.loc),

	);

	return $data{$o};
}

sub get_handler {

	my $o    = shift;
	my %data = (
		aga             => sub { ( split /:/, shift )[1] },
		planted_metazoa => sub { return 'U' },
		dmel            => sub {
			$_ = shift;
			if (/^arm_(\S+)/) {
				return $1;
			}
			elsif (/^Arm(\S+)/) {
				return $1;
			}
			elsif (/^(\S+)/) {
				return $1;
			}
			else {
				return '??';
			}
		}
	);

	return $data{$o};
}

1;
