#!/usr/bin/perl -w
# $Id: bootstrap_mature.pl,v 1.6 2010/07/07 12:34:59 rnl14480 Exp $

use strict;
use lib qw(../lib/);

use BAOBAB::ChromosomeWrapper;
use BAOBAB::Chromosome;
use BAOBAB::OrganismWrapper;
use BAOBAB::Organism;
use BAOBAB::PrecursorWrapper;
use BAOBAB::Precursor;
use BAOBAB::Mature;
use BAOBAB::MatureWrapper;
use BAOBAB::Fasta;
use BAOBAB::Wrapper;
use DBI;
use MirBase::ParseName qw(parse_name);

my $dbh = DBI->connect(
	$BAOBAB::Wrapper::dsn, $BAOBAB::Wrapper::user,
	$BAOBAB::Wrapper::password, { RaiseError => 1, AutoCommit => 0 }
) || die "Could not connect to the database : $DBI::errstr\n";
$dbh->{mysql_auto_reconnect} = 1;

die "Usage: $0 data-dir\n" unless scalar(@ARGV) >= 1;

my $data      = shift;
my @organisms = qw( aga dmel planted_metazoa);

my $mature = $data . "mature.fa";

my $mfasta = BAOBAB::Fasta->new(
	filename       => $mature,
	header_handler => sub {
		local $_ = shift;
		return ( split /\s+/ )[0];
	},
);

my %mature_hash = ();
my @msequences  = $mfasta->sequences;

foreach my $mseq (@msequences) {
	my $info = parse_name( $mseq->id );
	push @{ $mature_hash{ lc( $info->{mature_id} ) } }, $mseq;
}

foreach my $org (@organisms) {

	print "Processing organism $org...\n";
	my $fasta   = get_fasta($org);
	my $loc     = get_loc($org);
	my $handler = get_handler($org);

	print STDERR "Processing Fasta file : $fasta\n";
	my $fobj = BAOBAB::Fasta->new(
		filename       => $fasta,
		header_handler => $handler
	);

	my $locfile;
	open( $locfile, $loc ) or die "Cannot open file $loc : $!\n";

	my $ow = BAOBAB::OrganismWrapper->new( dbh => $dbh );
	$ow->read( shortname => $org );

	while (<$locfile>) {
		chomp;
		my @fields      = split /:/;
		my $chr_nr      = $fields[1];
		my $strand      = $fields[2];
		my $start       = $fields[3];
		my $stop        = $fields[6];
		my $designation = $fields[7];

		my @seqobjs = $fobj->sequences;
		my $seqobj  = $seqobjs[$chr_nr];
		my $chr     = $seqobj->id;

		my $cw = BAOBAB::ChromosomeWrapper->new( dbh => $dbh );
		$cw->read( name => $chr, oid => $ow->wrap->id );

		my $pseq = $fobj->get_region(
			chr    => $chr,
			offset => $start,
			len    => $stop - $start + 1,
			strand => $strand
		);

		my $pw = BAOBAB::PrecursorWrapper->new( dbh => $dbh );
		my $p = $pw->read(
			chromosome => $cw->wrap,
			category   => 'known',
			start      => $start,
			stop       => $stop,
			strand     => $strand
		);

		my $info  = parse_name($designation);
		my $mseqs = $mature_hash{ lc( $info->{mature_id} ) };
		my @mseqs = @$mseqs if defined $mseqs;
		warn "Precursor $designation does not have a mature sequence\n"
		  unless @mseqs;
		foreach my $mseq (@mseqs) {
			my $msequence = $mseq->sequence;
			my $position  = 0;
			my $written   = 0;
			while (
				( $position = index( $pseq, $msequence, $position ) ) != -1 )
			{
				warn "Additional mature sequence @ $designation\n"
				  if $written > 0;
				my $mobj = BAOBAB::Mature->new(
					precursor   => $p,
					position    => $position,
					length      => length($msequence),
					designation => $mseq->id
				);
				my $mw =
				  BAOBAB::MatureWrapper->new( dbh => $dbh, object => $mobj );
				$mw->write;
				$position++;
				$written++;
			}

		}

	}
}

$dbh->disconnect;

sub get_fasta {
	my $o    = shift;
	my %data = (
		aga             => $data . q(aga.nfa),
		planted_metazoa => $data . q(planted_metazoa.nfa),
		dmel            => $data . q(dmel.nfa),
	);

	return $data{$o};
}

sub get_loc {
	my $o    = shift;
	my %data = (
		aga             => $data . q(aga.loc),
		planted_metazoa => $data . q(planted_metazoa.loc),
		dmel            => $data . q(dmel.loc),
	);

	return $data{$o};
}

sub get_handler {

	my $o    = shift;
	my %data = (
		aga             => sub { ( split /:/, shift )[1] },
		planted_metazoa => sub { return 'U' },
		dmel            => sub {
			$_ = shift;
			if (/^arm_(\S+)/) {
				return $1;
			}
			elsif (/^Arm(\S+)/) {
				return $1;
			}
			elsif (/^(\S+)/) {
				return $1;
			}
			else {
				return '??';
			}
		}
	);

	return $data{$o};
}

1;
