#!/usr/bin/perl -w

use strict;
use lib qw (../lib/);
use BAOBAB::RNA2D;
use Fcntl qw(:seek);
use RNA;

# NFA file is a FastA file where the sequence is in a single line
die "Usage: $0 GFFfile NFAfile\n" unless @ARGV > 1;
my $fname = $ARGV[0];

my $file;
open( $file, $fname ) or die "Cannot open $fname : $!\n";

my $genfile = $ARGV[1];
my $finame  = $genfile . ".idx";

my $genome;
open( $genome, $genfile ) or die "Cannot open $genfile : $!\n";

if ( !-e $finame ) {
	warn "Computing index...\n";
	my @seq = compute_index($fname);
	my $idx;
	open( $idx, ">$finame" ) or die "Cannot open $finame : $!\n";
	print $idx $_->[0] . ":" . $_->[1] . "\n" for @seq;
	close($idx);
}

my @index = load_index($finame);
warn "Loaded " . scalar(@index) . " entries\n";

# aga
# my @cord = qw(X 2R 2L 3R 3L UNKN);

# dmel
my @cord = qw( X 2R 2L 3R 3L 4 U Uextra XHet 2RHet 2LHet 3RHet 3LHet YHet);

my %map;
my $place = 0;
$map{$_} = $place++ for (@cord);

my $mid = -1;
while (<$file>) {
	next if /^#/;
	chomp;
	my @f = split /\s+/;
	my ( $chr, $start, $stop, $strand, $name ) = @f[ 0, 3, 4, 6, 9 ];
	my ($mirna) = ( $name =~ /ID="([^"]+)";/ );
	$strand = $strand eq '+' ? 'F' : 'R';

	$start--;
	$stop--;    # we start at 0

	my $seq = $map{$chr};

	my $region = get_region(
		idx      => \@index,
		file     => $genome,
		sequence => $seq,
		offset   => $start,
		len      => $stop - $start + 1
	);

	if ( $strand eq 'R' ) {
		$region = reverse_complement($region);
	}

	$region =~ y/tT/uU/;
	my ( $str, $mfe ) = RNA::fold($region);

	my $rna = BAOBAB::RNA2D->new( sequence => $region, structure => $str );
	my ( $init, $halt ) = $rna->locate_terminal_loop;

	$mid++;

	# determine START STOP of pre-miRNA wrt to START/STOP of largest stem loop

	my $sl = $rna->largest_stemloop;
	if ( $strand eq 'F' ) {
		$init += $start;
		$halt += $start;
	}
	else {
		$init = $stop - $init;
		$halt = $stop - $halt;
		( $init, $halt ) = ( $halt, $init );
	}
	my ( $commencement, $finish );

	if ( $strand eq 'F' ) {
		$commencement = $start + $sl->{start};
		$finish       = $start + $sl->{stop};
	}
	else {
		$commencement = $stop - $sl->{stop};
		$finish       = $stop - $sl->{start};
	}

	#print "$mid:$seq:$strand:$commencement:$init:$halt:$finish:$mirna\n";
	print "$mid:$seq:$strand:$start:$init:$halt:$stop:$mirna\n";

}

sub reverse_complement {
	my $seq = shift;
	my @seq = split //, $seq;
	@seq = reverse(@seq);
	$seq = join( '', @seq );
	$seq =~ y/ATGCatgc/TACGtacg/;
	return $seq;
}

sub get_region {
	no strict 'subs';
	my %a      = @_;
	my $idx    = $a{idx};
	my $len    = $a{len};
	my $offset = $a{offset};
	my $seq    = $a{sequence};
	my $file   = $a{file};

	my $b = $idx->[$seq];
	my ( $start, $stop ) = @$b;
	if ( $offset > $stop - $start ) {
		die "Offset too big\n";
	}
	seek( $file, $start,  SEEK_SET );
	seek( $file, $offset, SEEK_CUR );
	my $region;
	read( $file, $region, $len );
	return $region;
}

sub load_index {
	my @index  = ();
	my $finame = shift;
	my $idx;

	open( $idx, $finame ) or die "Cannot open $finame : $!\n";
	while (<$idx>) {
		chomp;
		my ( $start, $stop ) = split /:/;
		push @index, [ $start, $stop ];
	}
	return wantarray ? @index : \@index;
}

sub compute_index {
	my $fname = shift;
	my ($file);
	open( $file, $fname ) or die "Cannot read $fname : $!\n";

	my $state = 0;
	my @sequences;
	my ( $start, $stop );
	while ( !eof($file) ) {
		my ($c);
		read( $file, $c, 1 );
		if ( $c eq '>' ) {
			if ( $state == 3 ) {    # No \n between seqs
				push @sequences, [ $start, $stop ];
			}
			$state = 1;
		}
		elsif ( $state == 1 ) {
			if ( $c eq "\n" ) {
				$state = 2;
				$start = tell($file);
			}
		}
		elsif ( $state == 2 && $c eq "\n" ) {
			$state = 3;
			$stop  = tell($file);    #just in case eof follows
		}
		elsif ( $state == 3 && $c eq "\n" ) {
			$state = 0;
			$stop  = tell($file);
			push @sequences, [ $start, $stop ];
		}
		elsif ( $state == 3 && $c ne "\n" ) {
			$state = 2;
		}
	}

	# in case there is no empty line at the end
	if ( defined $start && defined $stop ) {
		push @sequences, [ $start, $stop ];
	}

	close($file);

	return wantarray ? @sequences : \@sequences;

}

close($file);
close($genome);
